/* ----------------------------------------------------------------------------
 * GKrellM PMU plugin
 * Battery monitor for Apple Powerbooks running Linux/PPC
 * Copyright (c) 2003 Matthias Grimm
 * Version:  2.0
 *
 * based on gkrellm-pmu version 1.4 from Joseph Palani Garcia
 * last changed on 22. August 2000 from Joseph Palani Garcia
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 * ----------------------------------------------------------------------------
 */

#include "gkrellm-pmu.h"
#include "../pixmap/battery.xpm"   /* default pixmaps for battery panels */

struct pmuinfo pmu_data;

void
pmu_update ()
{
	struct pmuinfo *base = &pmu_data;
	GkrellmTicks *GKTicks;
#ifdef ENABLE_PBB	
	char msgbuffer[100];
	struct pbbmessage *msg = (struct pbbmessage *) msgbuffer;
#endif
	GKTicks = gkrellm_ticks();
	if (GKTicks->two_second_tick) {
#ifdef ENABLE_PBB
		if ((ipc_receive (msgbuffer, sizeof(msgbuffer))) == 0) {
			base->pbbBattPresent = tagfind (msg->taglist, TAG_BATTERYPRESENT, -1);
			singletag_to_server (READVALUE, TAG_BATTERYPRESENT, -1); 
		}
#endif
		if ((procpmu_update_batteryinfo (base))) { /* read /proc/pmu/battery_% */
			gkrellm_panel_destroy(base->panel);    /* battery config has changed */
			pmupanel_create (base, base->vbox, 1); /* so rebuild the panel */
			pmuchart_visibility (base);  /* set chart visible if usefull */
		}
		pmuchart_update (base);
		pmupanel_update (base);
	}
}

void
pmu_create (GtkWidget *vbox, gint first_create)
{
	struct pmuinfo *base = &pmu_data;

	if (first_create) {
		base->vbox = vbox;  /* save vbox for later rebuilding the panel */
	   	gkrellm_load_piximage ("battery", battery_xpm,
			&base->symbols, PMU_STYLE_NAME);
		gkrellm_scale_piximage_to_pixmap (base->symbols,
			&base->symbols_pixmap, &base->symbols_mask, 0, 0);
	}
	
#ifdef ENABLE_PBB
	init_libpbb ();
	if (ipc_init (LIBMODE_CLIENT, 0) == 0) { /* setup IPC subsystem for clients */
		base->pbbSupport = 1;
		singletag_to_server (READVALUE, TAG_BATTERYPRESENT, -1); 
	}
#endif
	gkrellm_disable_plugin_connect(base->monitor, gkrellm_exit_plugin);

	pmuchart_create (base, vbox, first_create);
	pmuchart_update (base);
	pmupanel_create (base, vbox, first_create);
	pmupanel_update (base);

/*	if (!first_create)*/
		pmuchart_visibility (base);
}

static GkrellmMonitor pmu_mon = {
	PMU_CONFIG_NAME,/* Name, for config tab.        */
	0,              /* Id,  0 if a plugin           */
	pmu_create,	 /* The create_plugin() function */
	pmu_update,      /* The update_plugin() function */
	pmuconfig_create, /* The create_plugin_tab() config function */
	pmuconfig_apply,  /* The apply_plugin_config() function      */

	pmuconfig_save,  /* The save_plugin_config() function  */
	pmuconfig_load,  /* The load_plugin_config() function  */
	PMU_CONFIG_KEY, /* config keyword                     */

	NULL,           /* Undefined 2  */
	NULL,           /* Undefined 1  */
	NULL,           /* Undefined 0  */

	PMU_PLACEMENT,  /* Insert plugin before this monitor.       */
	NULL,           /* Handle if a plugin, filled in by GKrellM */
	NULL            /* path if a plugin, filled in by GKrellM   */
};

GkrellmMonitor *
gkrellm_init_plugin(void)
{
	struct pmuinfo *base = &pmu_data;
	int n;

#ifdef ENABLE_NLS
/*	setlocale(LC_ALL, ""); */
	bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
	textdomain (PACKAGE);
	bind_textdomain_codeset (PACKAGE, "UTF-8");
#endif

	procpmu_init_pmuinfo (base);    /* initialize PMU related data */
	base->pbbSupport     =  0;      /* no PBButtons support by default */
	base->pbbBattPresent = -1;      /* PBB battery status not available */
	pmuconfig_init (&base->config); /* set up default config */
	base->monitor = &pmu_mon;       /* initialize Gkrellm related date */
                                        /* in struct pmu_data */
	
	g_atexit (gkrellm_exit_plugin);
	
	for (n=0; n < MAXBATTERIES; n++)
		base->battery[n].displaymode = DM_NONE + 1;
	base->panel          = NULL;
	base->chart          = NULL;
	base->chartconfig    = NULL;
	base->symbols        = NULL;
	base->symbols_pixmap = NULL;
	base->symbols_mask   = NULL;
	base->style_id       = gkrellm_add_meter_style (base->monitor, PMU_STYLE_NAME);

	procpmu_get_pmuinfo (base);        /* read /proc/pmu/info */
	procpmu_update_batteryinfo (base); /* read /proc/pmu/battery_% */

	return base->monitor;
}

void
gkrellm_exit_plugin (void)
{
	struct pmuinfo *base = &pmu_data;
	
	gkrellm_chart_hide (base->chart, 1);
#ifdef ENABLE_PBB
	ipc_exit ();
	exit_libpbb ();
#endif
}

